<?php

/**
 * @file
 * Notifications receiver.
 */

use Commerce\Adyen\Payment\Exception\NotificationException;

/**
 * {@inheritdoc}
 *
 * @see commerce_adyen_menu()
 *
 * @internal
 */
function commerce_adyen_notification() {
  $dump = function ($variable) {
    ob_start();
    var_dump($variable);

    return ob_get_clean();
  };

  $watchdog = function (\Exception $e = NULL) use ($dump) {
    $message = "Notification has been received. <pre>@data</pre>";
    $arguments = [
      '@data' => $dump($_POST),
    ];

    if (NULL !== $e) {
      $message = 'Notification has been handled' . ($e instanceof NotificationException ? ' wrongly' : '') . ' and exception been thrown: <pre>@exception</pre><pre>@data</pre>';
      $arguments['@exception'] = $dump($e);
    }

    watchdog(COMMERCE_ADYEN_PAYMENT_METHOD, $message, $arguments);
  };

  try {
    // POST request should be here. No sense to let Drupal continue working.
    if (empty($_POST)) {
      throw new \RuntimeException('Adyen notification is empty.');
    }

    $data = (object) $_POST;
    $order = commerce_order_load_by_number($data->merchantReference);

    // Order cannot be located/loaded. No sense to let Drupal continue working.
    if (FALSE === $order) {
      throw new \RuntimeException('Authorisation request made with incorrect "merchantReference" property which must be a valid order number.');
    }

    // Yeah, Adyen, you are nice guy! Let's send us something similar
    // in that spirit. We will process everything.
    // The "success" or "live" properties will be strings with "true"
    // or "false" values.
    foreach ($data as $property => $value) {
      switch (drupal_strtolower($value)) {
        case 'true':
          $data->$property = TRUE;
          break;

        case 'false':
          $data->$property = FALSE;
          break;
      }
    }

    // Log the notification (data from POST request).
    $watchdog();

    // Notify subscribed modules about notification.
    module_invoke_all(__FUNCTION__, $data->eventCode, $order, $data);
  }
  // Something went terribly wrong - notification should be put in a queue.
  catch (NotificationException $e) {
    $watchdog($e);
    exit();
  }
  // All other exceptions does not mean that we didn't handle notification.
  catch (\Exception $e) {
    $watchdog($e);
  }

  // Return "[accepted]" to Adyen. This is essential to let it know that
  // notification has been received. If Adyen do NOT receive "[accepted]"
  // then it'll try to send it again which will put all other notification
  // in a queue.
  exit('[accepted]');
}
